// B-tree map of numbers to numbers -*- c++ -*-

#ifndef BTREE_H_
# define BTREE_H_
# ifdef __GNUC__
#  pragma interface
# endif // __GNUC__

# include "file.h"

/** @file BTree.h
 * B-tree map of numbers to numbers
 */

/** Size of the b-tree nodes (must be at least 8 and divisible by 4) */
# define BTREE_SIZE 512

/** B-tree map of numbers to numbers */
class BTree
{
public:
  /** Type of the numbers to be stored */
  typedef unsigned item_t;
  /** Constructor
   * @param file	Disk file containing the B-tree
   */
  explicit BTree (file_t file);
private:
  /** Copy constructor */
  BTree (const class BTree& old);
  /** Assignment operator */
  class BTree& operator= (const class BTree& old);
public:
  /** Destructor */
  ~BTree ();
  /** Search a tree
   * @param key		the key to be sought
   * @return		the values (item 0: amount of the values)
   */
  item_t* search (item_t key) const;
  /** Insert an item to the tree
   * @param key		the key
   * @param value	the value
   */
  void insert (item_t key, item_t value);

private:
  /** The disk file containing the B-tree */
  file_t myFile;
# ifndef USE_MMAP
  /** Copy of the root node of the B-tree */
  item_t myRoot[BTREE_SIZE];
# endif // USE_MMAP
};

#endif // BTREE_H_
