/* Efficient file abstraction layer -*- c++ -*- */

#ifndef FILE_H_
# define FILE_H_

/** @file file.h
 * Efficient file abstraction layer.
 * Use mmap(2) and friends on Unix systems when USE_MMAP is defined.
 * As a portable fall-back method, use <cstdio> FILE pointers.
 */

# include <cstdio>

# if defined __unix && defined USE_MMAP
#  include <unistd.h>
#  include <sys/mman.h>
/** File handle */
typedef struct
{
  int fd;	/**< UNIX file descriptor */
  long len;	/**< used file length in bytes */
  long alloc;	/**< allocated file length in bytes */
#  ifdef __sun
  caddr_t addr;	/**< address of the mapped memory area */
#  else // __sun
  void* addr;	/**< address of the mapped memory area */
#  endif // __sun
} file_t;
# else // __unix && USE_MMAP
#  undef USE_MMAP
/** File handle */
typedef FILE* file_t;
# endif // __unix && USE_MMAP

#endif // FILE_H_
